clear all

% parameters of the Hamiltonian

e_d = 2; % On-site energy 
U = 10; % Interaction

% Bath parameters
Gamma = 0.01; %bath-impurity tunnelling rate
T = 10; % Temperature
mu = 1; % Chemical potential

t_max = 10/Gamma;
global e_d U Gamma  T mu


options = odeset('RelTol',1e-4,'AbsTol',1e-4);

for i = 1:100;
    
%initial condition 
rho_In = rand(1,4);
rho_In = rho_In/sum(rho_In);


[t,rho] = ode45(@Anderson_L,[0 t_max],rho_In,options);

plot(t*Gamma,rho);
hold on

end

% Calculation of the analytical stationary states

fp = fermi(e_d-mu,T);
fm = 1-fp;
fUp = fermi(e_d+U-mu,T);
fUm = 1-fUp;

rho_Stat = [1,fp/fm, fp/fm, fp/fm*fUp/fUm];

rho_Stat = rho_Stat/sum(rho_Stat);

plot(t*Gamma,repmat(rho_Stat,length(t),1),'k.-')

%%%%% Graphical makeup %%%%%%%%%%%%%%%%%%%%%%%%%%
ylim([0 1])
title('Anderson impurity model','Fontsize',16)
xlabel('time [1/\gamma]','Fontsize',16)
ylabel('Occupation','Fontsize',16)
set(gca,'Fontsize',16)

legend('P_0','P_{1\uparrow}','P_{1\downarrow}','P_2')

hold off