% clear all

% Program that calculates the time evolution of the 
% density matrix for an Anderson impurity model

% parameters of the Impurity
e_d = 2; % On-site energy 
U = 3;   % Interaction

% Bath parameters
T = 1;        % Temperature
mu = 10; % Chemical potential

% Parameters of the coupling
Gamma = 0.01; %bath-impurity tunnelling rate

% Energy, particle number and Free energy of 
% the many body eigenstates 
En = [0, e_d, e_d, 2*e_d+U];
N = [0 1 1 2];
F = En - N*mu;


% Discretization parameters
t_max = 10/Gamma;
Nt = 100;
t_vec = linspace(0,t_max,Nt);


% Usefull f+ and f- components of the Liouvillean
fp = fermi(e_d-mu,T);
fm = 1-fp;
fUp = fermi(e_d+U-mu,T);
fUm = 1-fUp;


% the Liouvillean
L = Gamma*[-2*fp,fm,fm,0;...
           fp,-fm-fUp,0,fUm;...
           fp,0,-fm-fUp,fUm;...
           0,fUp,fUp,-2*fUm];

for i = 1:50
    
%initial condition 
rho_In = rand(1,4);
rho_In = rho_In/sum(rho_In);


for i = 1:Nt
  t = t_vec(i);
  
  rho(i,:) = expm(L*t)*rho_In'; 
      
end
figure(1)
subplot(1,2,1)
plot(t_vec*Gamma,rho)
%%%%% Graphical makeup figure 1%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ylim([0 1])
title('Anderson impurity model','Fontsize',16)
xlabel('\gamma time','Fontsize',16)
ylabel('Occupation','Fontsize',16)
set(gca,'Fontsize',16)
legend('P_0','P_{1\uparrow}','P_{1\downarrow}','P_2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hold on

end
% Calculation of the analytical stationary states

rho_Stat = [1,fp/fm, fp/fm, fp/fm*fUp/fUm];
rho_Stat = rho_Stat/sum(rho_Stat);

plot(t*Gamma,rho_Stat,'ko')
hold off





l = (1/200:1/100:1)'*2*pi;
x = sin(l);
y = cos(l);
pause


figure(1)
subplot(1,2,2)

for i = 1:Nt
    for j = 1:4
        colore = 'r'
        plot([2*N(j)-0.5, 2*N(j)+0.5], [F(j) F(j)],'k','Linewidth',2)
        hold on
        if j == 2
            colore = 'g'    
        end
        fill(rho(i,j)/2*x+2*N(j),rho(i,j)/2*y+En(j)-N(j)*mu,colore)
        axis equal
        ylim([min(F)-1 max(F)+1])
        xlim([2*min(N)-1  2*max(N)+1])
        xlabel('Particle Number','Fontsize',16)
        ylabel('Free Energy [eV]','Fontsize',16)
        title(['\gamma t = ',num2str(t_vec(i)*Gamma, '%6.1f')])
        set(gca,'Xtick',[0 2 4],'XtickLabel',[0 1 2],'Fontsize',16)
    end
    pause(0.05)
    hold off
end
hold off
       
